##############################################################################
#
# Copyright (c) 2003 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""SoapFix

  The SoapFix product refines the current Zope (2.6.3, 2.7.0) behavior of
  treating all HTTP POST requests with text/xml content bodies as XML-RPC
  requests.

  The default Zope behavior makes it impossible to write objects or scripts
  to handle SOAP requests, which are also sent via POST using text/xml.
  This product makes Zope a little smarter - if a POST with a content type
  of text/xml appears to be a SOAP envelope, it will be passed through to
  the application and not handed to the xml-rpc handlers.
"""

from ZPublisher.HTTPRequest import HTTPRequest
from StringIO import StringIO

# This product works by replacing the default processInputs method of
# ZPublisher's HTTPRequest with a custom version. Our custom method
# tries to determine if the request is a SOAP request before committing
# to using xml-rpc. If it looks like a SOAP request, we change the
# request content-type to 'text/xml-soap' temporarily and call the
# original processInputs (retained as _oldProcessInputs), which will
# now leave the xml payload alone allowing app code to perform SOAP
# processing.


def processInputs(self, *args, **kw):
    environ = self.environ
    method = environ.get('REQUEST_METHOD')
    c_type = environ.get('CONTENT_TYPE')

    if ((method == 'POST') and (c_type == 'text/xml')):
        clues = ('/soap/envelope', '/soap-envelope')
        is_soap = 0

        # This is not ideal if the request body is large, but most SOAP
        # requests are fairly small (and the body will need to be brought
        # into memory anyway to do the soap xml parsing).

        body = self.stdin.read()
        self.stdin = StringIO(body)
        self._file = self.stdin
        for item in clues:
            if body.find(item) > -1:
                is_soap = 1
                break

        if is_soap:
            self.environ['CONTENT_TYPE'] = 'text/xml-soap'
            r = self._realProcessInputs(*args, **kw)
            self.environ['CONTENT_TYPE'] = 'text/xml'
            return r

    return self._realProcessInputs(*args, **kw)


def initialize(context):
    HTTPRequest._realProcessInputs = HTTPRequest.processInputs
    HTTPRequest.processInputs = processInputs
